@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2008 Oracle.  All rights reserved.
  *
- * $Id: LatchException.java,v 1.17.2.1 2007/02/01 14:49:46 cwl Exp $
+ * $Id: LatchException.java,v 1.20 2008/01/07 14:28:50 cwl Exp $
  */
 
 package com.sleepycat.je.latch;
@@ -16,10 +16,6 @@ import com.sleepycat.je.DatabaseException;
 
 public class LatchException extends DatabaseException {
 
-    public LatchException() {
-	super();
-    }
-
     public LatchException(String message) {
 	super(message);
     }
