@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2008 Oracle.  All rights reserved.
  *
- * $Id: LatchNotHeldException.java,v 1.15.2.1 2007/02/01 14:49:46 cwl Exp $
+ * $Id: LatchNotHeldException.java,v 1.18 2008/01/07 14:28:50 cwl Exp $
  */
 
 package com.sleepycat.je.latch;
@@ -13,9 +13,6 @@ package com.sleepycat.je.latch;
  * on it that assumes it is held.
  */
 public class LatchNotHeldException extends LatchException {
-    public LatchNotHeldException() {
-	super();
-    }
 
     public LatchNotHeldException(String message) {
 	super(message);
