@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002,2008 Oracle.  All rights reserved.
  *
- * $Id: DBINReference.java,v 1.14.2.1 2007/02/01 14:49:51 cwl Exp $
+ * $Id: DBINReference.java,v 1.18 2008/05/13 01:44:53 cwl Exp $
  */
 
 package com.sleepycat.je.tree;
@@ -11,14 +11,16 @@ package com.sleepycat.je.tree;
 import com.sleepycat.je.dbi.DatabaseId;
 
 /**
- * A class that embodies a reference to a DBIN that does not rely on a
- * java reference to the actual DBIN.
+ * A class that embodies a reference to a DBIN that does not rely on a Java
+ * reference to the actual DBIN.
  */
 public class DBINReference extends BINReference {
     private byte[] dupKey;
 
-    DBINReference(long nodeId, DatabaseId databaseId,
-                  byte[] idKey, byte[] dupKey) {
+    DBINReference(long nodeId,
+                  DatabaseId databaseId,
+                  byte[] idKey,
+                  byte[] dupKey) {
 	super(nodeId, databaseId, idKey);
 	this.dupKey = dupKey;
     }
@@ -35,4 +37,3 @@ public class DBINReference extends BINReference {
 	return super.toString() + " dupKey=" + Key.dumpString(dupKey, 0);
     }
 }
-
